/*****************************************************************************
 * Copyright (c) 2020, rrcc FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "CalcImm.h"

std::vector<uint8_t> Instruction::CalcImm::toBytes() const {
    std::vector<uint8_t> bytes;

    bytes.emplace_back((uint8_t) getCommand() | ((uint8_t) operation << 2) | addressOffset);

    bytes.emplace_back((addressDestination >> 16) & 0xFF);
    bytes.emplace_back((addressDestination >> 8) & 0xFF);
    bytes.emplace_back(addressDestination & 0xFF);

    bytes.emplace_back(0x00);

    bytes.emplace_back((addressSource >> 16) & 0xFF);
    bytes.emplace_back((addressSource >> 8) & 0xFF);
    bytes.emplace_back(addressSource & 0xFF);

    bytes.emplace_back((value >> 24) & 0xFF);
    bytes.emplace_back((value >> 16) & 0xFF);
    bytes.emplace_back((value >> 8) & 0xFF);
    bytes.emplace_back(value & 0xFF);

    return bytes;
}


std::unique_ptr<Instruction::Instruction> Instruction::CalcImm::fromAndTokens(
        const Instruction::parseOperatorResult &op, const std::vector<Token> &tokens) {
    if (tokens.at(2).getType() == Token::Type::Immediate) {
        return std::make_unique<CalcImm>(Calc::Operation::AND, op.offset1,
                                         tokens.at(0).getNumericValue(),
                                         tokens.at(1).getNumericValue(),
                                         tokens.at(2).getNumericValue());
    } else {
        return std::make_unique<Calc>(Calc::Operation::AND, op.offset1,
                                      tokens.at(0).getNumericValue(),
                                      tokens.at(1).getNumericValue(),
                                      tokens.at(2).getNumericValue());
    }
}

std::unique_ptr<Instruction::Instruction> Instruction::CalcImm::fromOrTokens(
        const Instruction::parseOperatorResult &op, const std::vector<Token> &tokens) {
    if (tokens.at(2).getType() == Token::Type::Immediate) {
        return std::make_unique<CalcImm>(Calc::Operation::OR, op.offset1,
                                         tokens.at(0).getNumericValue(),
                                         tokens.at(1).getNumericValue(),
                                         tokens.at(2).getNumericValue());
    } else {
        return std::make_unique<Calc>(Calc::Operation::OR, op.offset1,
                                      tokens.at(0).getNumericValue(),
                                      tokens.at(1).getNumericValue(),
                                      tokens.at(2).getNumericValue());
    }
}

std::unique_ptr<Instruction::Instruction> Instruction::CalcImm::fromAddTokens(
        const Instruction::parseOperatorResult &op, const std::vector<Token> &tokens) {
    if (tokens.at(2).getType() == Token::Type::Immediate) {
        return std::make_unique<CalcImm>(Calc::Operation::ADD, op.offset1,
                                         tokens.at(0).getNumericValue(),
                                         tokens.at(1).getNumericValue(),
                                         tokens.at(2).getNumericValue());
    } else {
        return std::make_unique<Calc>(Calc::Operation::ADD, op.offset1,
                                      tokens.at(0).getNumericValue(),
                                      tokens.at(1).getNumericValue(),
                                      tokens.at(2).getNumericValue());
    }
}

std::unique_ptr<Instruction::Instruction> Instruction::CalcImm::fromSubTokens(
        const Instruction::parseOperatorResult &op, const std::vector<Token> &tokens) {
    if (tokens.at(2).getType() == Token::Type::Immediate) {
        return std::make_unique<CalcImm>(Calc::Operation::SUB, op.offset1,
                                         tokens.at(0).getNumericValue(),
                                         tokens.at(1).getNumericValue(),
                                         tokens.at(2).getNumericValue());
    } else {
        return std::make_unique<Calc>(Calc::Operation::SUB, op.offset1,
                                      tokens.at(0).getNumericValue(),
                                      tokens.at(1).getNumericValue(),
                                      tokens.at(2).getNumericValue());
    }
}

std::unique_ptr<Instruction::Instruction> Instruction::CalcImm::fromLShiftTokens(
        const Instruction::parseOperatorResult &op, const std::vector<Token> &tokens) {
    if (tokens.at(2).getType() == Token::Type::Immediate) {
        return std::make_unique<CalcImm>(Calc::Operation::LSHIFT, op.offset1,
                                         tokens.at(0).getNumericValue(),
                                         tokens.at(1).getNumericValue(),
                                         tokens.at(2).getNumericValue());
    } else {
        return std::make_unique<Calc>(Calc::Operation::LSHIFT, op.offset1,
                                      tokens.at(0).getNumericValue(),
                                      tokens.at(1).getNumericValue(),
                                      tokens.at(2).getNumericValue());
    }
}

std::unique_ptr<Instruction::Instruction> Instruction::CalcImm::fromRShiftTokens(
        const Instruction::parseOperatorResult &op, const std::vector<Token> &tokens) {
    if (tokens.at(2).getType() == Token::Type::Immediate) {
        return std::make_unique<CalcImm>(Calc::Operation::RSHIFT, op.offset1,
                                         tokens.at(0).getNumericValue(),
                                         tokens.at(1).getNumericValue(),
                                         tokens.at(2).getNumericValue());
    } else {
        return std::make_unique<Calc>(Calc::Operation::RSHIFT, op.offset1,
                                      tokens.at(0).getNumericValue(),
                                      tokens.at(1).getNumericValue(),
                                      tokens.at(2).getNumericValue());
    }
}

std::unique_ptr<Instruction::Instruction> Instruction::CalcImm::fromARShiftTokens(
        const Instruction::parseOperatorResult &op, const std::vector<Token> &tokens) {
    if (tokens.at(2).getType() == Token::Type::Immediate) {
        return std::make_unique<CalcImm>(Calc::Operation::ARSHIFT, op.offset1,
                                         tokens.at(0).getNumericValue(),
                                         tokens.at(1).getNumericValue(),
                                         tokens.at(2).getNumericValue());
    } else {
        return std::make_unique<Calc>(Calc::Operation::ARSHIFT, op.offset1,
                                      tokens.at(0).getNumericValue(),
                                      tokens.at(1).getNumericValue(),
                                      tokens.at(2).getNumericValue());
    }
}