/*****************************************************************************
 * Copyright (c) 2020, rrcc FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "Poll.h"

std::vector<uint8_t> Instruction::Poll::toBytes() const {
    std::vector<uint8_t> bytes;

    bytes.emplace_back((uint8_t) getCommand() | (equality << 2) | address.offset);

    bytes.emplace_back((address.address >> 16) & 0xFF);
    bytes.emplace_back((address.address >> 8) & 0xFF);
    bytes.emplace_back(address.address & 0xFF);

    bytes.emplace_back((value >> 24) & 0xFF);
    bytes.emplace_back((value >> 16) & 0xFF);
    bytes.emplace_back((value >> 8) & 0xFF);
    bytes.emplace_back(value & 0xFF);

    bytes.emplace_back((mask >> 24) & 0xFF);
    bytes.emplace_back((mask >> 16) & 0xFF);
    bytes.emplace_back((mask >> 8) & 0xFF);
    bytes.emplace_back(mask & 0xFF);


    bytes.emplace_back((maxRetry >> 8) & 0xFF);
    bytes.emplace_back(maxRetry & 0xFF);


    bytes.emplace_back((retryInterval >> 8) & 0xFF);
    bytes.emplace_back(retryInterval & 0xFF);

    bytes.emplace_back(0x00);

    bytes.emplace_back((jumpAddress >> 16) & 0xFF);
    bytes.emplace_back((jumpAddress >> 8) & 0xFF);
    bytes.emplace_back(jumpAddress & 0xFF);

    return bytes;
}

std::unique_ptr<Instruction::Instruction>
Instruction::Poll::fromEqualTokens(const Instruction::Instruction::parseOperatorResult &op,
                                   const std::vector<Token> &tokens) {
    return std::make_unique<Poll>(
            AddressWithOffset{tokens.at(0).getNumericValue(), op.offset1}, 1,
            tokens.at(1).getNumericValue(),
            tokens.at(2).getNumericValue(),
            tokens.at(3).getNumericValue(),
            tokens.at(4).getNumericValue(),
            tokens.at(5).getNumericValue());
}

std::unique_ptr<Instruction::Instruction>
Instruction::Poll::fromNonEqualTokens(const Instruction::Instruction::parseOperatorResult &op,
                                      const std::vector<Token> &tokens) {
    return std::make_unique<Poll>(
            AddressWithOffset{tokens.at(0).getNumericValue(), op.offset1}, 1,
            tokens.at(1).getNumericValue(),
            tokens.at(2).getNumericValue(),
            tokens.at(3).getNumericValue(),
            tokens.at(4).getNumericValue(),
            tokens.at(5).getNumericValue());
}