/*****************************************************************************
 * Copyright (c) 2020, rrcc FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/


#include "Call.h"
#include "../Return.h"
#include "../Jump.h"
#include "../Copy.h"
#include "../Write.h"
#include "Stack.h"


namespace Instruction {
    std::vector<uint8_t> Call::toBytes() const {
        std::vector<uint8_t> v = Stack::PushImm(getAddress() + getByteSize()).toBytes();

        if(address == 0){
            auto b = (Return(addressWithOffset)).toBytes();
            v.insert(v.end(), b.begin(), b.end());
        }else{
            auto b = (Jump(address)).toBytes();
            v.insert(v.end(), b.begin(), b.end());
        }
        return v;
    }


    std::unique_ptr<Instruction> Call::fromTokens(const Instruction::Instruction::parseOperatorResult &op,
                                                 const std::vector<Token> &tokens) {
        if (tokens.at(0).getType() == Token::Type::CodeLocation || tokens.at(0).getType() == Token::Type::CodeLabel) {
            return std::make_unique<Call>(tokens.at(0).getNumericValue());
        } else {
            return std::make_unique<Call>(AddressWithOffset{tokens.at(0).getNumericValue(), op.offset1});
        }
    }

    Call::Call(const AddressWithOffset &a) {
        addressWithOffset = a;
        address = 0;
    }

    Call::Call(uint32_t directAddress) {
        addressWithOffset = {0, 0};
        address = directAddress;
    }

    std::vector<uint8_t> Ret::toBytes() const {
        std::vector<uint8_t> v;
        for (auto &i : instructions) {
            auto b = i->toBytes();
            v.insert(v.end(), b.begin(), b.end());
        }
        return v;
    }


    std::unique_ptr<Instruction> Ret::fromTokens(const Instruction::Instruction::parseOperatorResult &op,
                                                  const std::vector<Token> &tokens) {
        if(!tokens.empty()){
            const auto& t = tokens.at(0);
            if(t.getType() == Token::Type::RegisterLocation){
                return std::make_unique<Ret>(AddressWithOffset{t.getNumericValue(), op.offset1});
            } else {
                return std::make_unique<Ret>(t.getNumericValue());
            }
        }
        return std::make_unique<Ret>();
    }

    Ret::Ret(const AddressWithOffset &address) {
        instructions.push_back(std::make_unique<Copy>(AddressWithOffset{(uint32_t)MgmtRegisters::std_RETURN_VALUE, 0}, address, 1));
        instructions.push_back(std::make_unique<Stack::PopImm>(1));
        instructions.push_back(std::make_unique<Return>(AddressWithOffset{0, Stack::stackOffsetRegister}));
    }

    Ret::Ret(uint32_t value) {
        instructions.push_back(std::make_unique<Write>(AddressWithOffset{(uint32_t)MgmtRegisters::std_RETURN_VALUE, 0}, std::vector<uint32_t>({value})));
        instructions.push_back(std::make_unique<Stack::PopImm>(1));
        instructions.push_back(std::make_unique<Return>(AddressWithOffset{0, Stack::stackOffsetRegister}));
    }

    Ret::Ret() {
        instructions.push_back(std::make_unique<Stack::PopImm>(1));
        instructions.push_back(std::make_unique<Return>(AddressWithOffset{0, Stack::stackOffsetRegister}));
    }

}

